/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */


#include "DefaultGNGContainer.hpp"
#include "../GNGCore/GNGNode.hpp"
#include "../GNGCore/GNGEdge.hpp"


//----------
DefaultGNGContainer::~DefaultGNGContainer()
{
  EdgeContainer::iterator it = m_edges.begin();
  while(it != m_edges.end())
    DeleteEdge(it);

  NodeContainer::iterator it2 = m_nodes.begin();
  while(it2 != m_nodes.end())
    DeleteNode(it2);
	  
  delete m_nodeFactory;
}

//----------
EdgeContainer & DefaultGNGContainer::GetEdges()
{
  return m_edges;
}

//----------
NodeContainer & DefaultGNGContainer::GetNodes()
{
  return m_nodes;
}

//----------
void DefaultGNGContainer::DeleteEdge(EdgeContainer::iterator & i)
{
  delete (*i);      // free memory
  m_edges.erase(i); // remove from reference
}

//----------
void DefaultGNGContainer::DeleteEdge(GNGEdge * e)
{
  EdgeContainer::iterator it = m_edges.begin();

  for(;it != m_edges.end(); it++)
  {
    if( (*it) == e )
    {
      DeleteEdge(it);
      return;
    }
  }  
}

//----------
void DefaultGNGContainer::DeleteNode(NodeContainer::iterator & i)
{
  delete (*i);      // free memory
  m_nodes.erase(i); // remove reference.
}


//----------
GNGEdge * DefaultGNGContainer::CreateEdge(GNGNode * n1, GNGNode * n2)
{
  GNGEdge * e = new GNGEdge(n1, n2);
  m_edges.push_back(e);
  return e;
}

//----------
GNGNode * DefaultGNGContainer::CreateNode()
{
  //GNGNode * node = new GNGNode();
  GNGNode * node = m_nodeFactory->NewNode();
  m_nodes.push_back(node);
  return node;
}


