/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __DEFAULTCONTAINER_H
#define __DEFAULTCONTAINER_H

#include "../GNGCore/IGNGContainer.hpp"
#include "../GNGCore/GNGNode.hpp"

class DefaultGNGContainer : public IGNGContainer
{
public:
  virtual ~DefaultGNGContainer();

  virtual void SetNodeFactory(GNGNodeFactory * f) { m_nodeFactory = f; }

  virtual EdgeContainer & GetEdges();
  virtual NodeContainer & GetNodes();

  /**
   * Deletes the edge at i from the internal container.
   * @param it the iterator positition of the edge to remove.
   */
  virtual void DeleteEdge(EdgeContainer::iterator & i);

  /**
   * Deletes the edge e from the internal container.
   * @param e the edge to delete.
   */
  virtual void DeleteEdge(GNGEdge * e);

  /**
   * Creates a new edge between n1 and n2, stores it in the internal edge container
   * and returns a pointer to the new edge.
   * @param n1 the first node to be connected.
   * @param n2 the second node to be connected.
   * @return the newly created edge.
   */
  virtual GNGEdge * CreateEdge(GNGNode * n1, GNGNode * n2);
  
  /**
   * Creates a new node, stores it in the internal node container
   * and returns a pointer to the new node.
   * @return the newly created node.
   */
  virtual GNGNode * CreateNode();

  /**
   * Deletes the node e from the internal container.
   * @param e the node to delete.
   */
  virtual void DeleteNode(NodeContainer::iterator & i);

protected:
  EdgeContainer      m_edges;
  NodeContainer      m_nodes;
  GNGNodeFactory *  m_nodeFactory;
};


#endif

