/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */


// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Vector.hpp>

#include <GNGCore/IGNGContainer.hpp>
#include <GNGCore/GNGNode.hpp>
#include <GNGCore/GNGEdge.hpp>
#include <RBF/RBFNode.hpp>

#include "SGNGNodeInserter.hpp"

#include <iostream>

using namespace std;

//----------
void SGNGNodeInserter::InsertNewNode()
{
  try 
	{
		GNGNode * nodeU = FindNodeWithLargestError();
		
		GNGNode * nodeV = FindNeighborWithLargestError(*nodeU);
		
		Vector & refU = nodeU->GetReferenceVector();
		Vector & refV = nodeV->GetReferenceVector();
		
		GNGNode * nodeR = m_container->CreateNode(); // CreateNode stores the node in the GNGContainer.
		nodeR->SetReferenceVector( refU + ((refV - refU) * m_paramNewNodePos) );
		
		GNGEdge * edge1 = m_container->CreateEdge(nodeU, nodeR); // CreateEdge stores the edge in the GNGContainer. 
		GNGEdge * edge2 = m_container->CreateEdge(nodeV, nodeR);
		
		edge1->SetAge(0);
		edge2->SetAge(0);
		
		m_container->DeleteEdge( nodeU->GetEdgeTo(*nodeV) );
		
		//recalculate widths
		((RBFNode*)nodeU)->SetWidth( CalculateMeanDistance(nodeU) );
		((RBFNode*)nodeV)->SetWidth( CalculateMeanDistance(nodeV) );
		((RBFNode*)nodeR)->SetWidth( CalculateMeanDistance(nodeR) );
		
		// according to the article on GNG by fritzke
		double errorU = nodeU->GetAccumulatedError() * m_paramErrorDecay;
		double errorV = nodeV->GetAccumulatedError() * m_paramErrorDecay;
		double errorR = NewError(errorU, errorV);
			
		double utilityR = (nodeU->GetUtility() + nodeV->GetUtility()) / 2.0;
		
		nodeU->SetAccumulatedError(errorU);
		nodeV->SetAccumulatedError(errorV);
		nodeR->SetAccumulatedError(errorR);
		nodeR->SetUtility(utilityR);
		
  }
  catch(Exception e)
  {
    cout << "InsertNewNode:: Node not inserted!" << endl;
  }

}


//-----------
double SGNGNodeInserter::CalculateMeanDistance(GNGNode * n)
{
  GNGNode & s = *n;
  double mean = 0.0;
  double num = 0.0;
  
  Vector & v = s.GetReferenceVector();
  EdgeContainer const & edges = s.GetEdges();
  EdgeContainer::const_iterator it = edges.begin();
  for(; it != edges.end(); it++)
  {
    num++;
    GNGNode & node = (*it)->GetOtherNode(s);
    Vector & v2 = node.GetReferenceVector();
    mean += fabs(v.Distance(v2));
  }    
			
  return (mean/num);
}
