/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __GNGCONTROL_H
#define __GNGCONTROL_H

#include <Control.hpp>
#include <Generators/IInputGenerator.hpp>
#include "GNGAlgorithm.hpp"

/**
* GNGGLUTRepresentation uses the control-object's
* Iterate method every time the CallBackIdleFunc is called.
* Placing the code separately in this object is a way
* of allowing GL independent code and state information.
* The GNGAlgorithm is iterated by a call to its Iterate() method.
*/
class GNGControl : public Control
{
public:
    /**
    * Constructs a new IdleObject with a reference to a GNGAlgorithm Object.
    * @param alg The GNGAlgorithm that will be iterated.
	*/
	GNGControl(GNGAlgorithm * alg, IInputGenerator * g) : m_alg(alg), m_generator(g) 
	{
		m_iteration = 0;
	}
	
	virtual ~GNGControl() 
	{ 
		delete m_generator; 
		delete m_alg; 
	}
	
	/**
	* The main iteration loop.
	* @return false when iteration should be ceased, true if continue.
	*/
	virtual bool Iterate()
	{
		if(IsStopCriteriaMet())
			return false;		
		
		if(IsReadyToIterate())
		{
			m_iteration++;
			m_alg->Iterate( m_generator->GetInput() );
			return true;
		}
		return false;
	}

protected:
    // never stop.
    virtual bool IsStopCriteriaMet()
    {
        return false;
    }
	
protected:
	GNGAlgorithm *     m_alg;
	IInputGenerator *  m_generator;
};


#endif
