/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include "../Exception.hpp"

#include "GNGEdge.hpp"
#include "GNGNode.hpp"

//---------
GNGEdge::GNGEdge(GNGNode * h1, GNGNode * h2)
{
  if(h1 == 0 || h2 == 0)
    throw Exception("GNGEdge::Constructor:: NULL argument(s)");
    
  m_node1 = h1;
  m_node2 = h2;
  m_age = 0;
  m_node1->AddEdge(*this);
  m_node2->AddEdge(*this);
}

//---------
GNGEdge::~GNGEdge()
{
  if(m_node1 != 0)
    m_node1->RemoveEdge(*this);
  if(m_node2 != 0)
    m_node2->RemoveEdge(*this);    
}  


//---------
// Throws Exception if node doesn't match
//
GNGNode const & GNGEdge::GetOtherNode(GNGNode const & node) const
{
	return const_cast<GNGEdge *>(this)->GetOtherNode(node);
}

//---------
// Throws Exception if node doesn't match
//
GNGNode & GNGEdge::GetOtherNode(GNGNode const & node) // throws Exception
{
  if(&node == m_node1)
    return *m_node2;

  if(&node == m_node2)
    return *m_node1;

  throw Exception("GetOtherNode:: Illegal node!");
  return *m_node1; // will not happen.
}


GNGNode & GNGEdge::GetNode1() const
{
	return *m_node1;
}

GNGNode & GNGEdge::GetNode2() const
{
	return *m_node2;
}


//---------
double GNGEdge::GetAge()
{
  return m_age;
}

//---------
void GNGEdge::SetAge(double d)
{
  m_age = d;
}




//---------
bool operator== (GNGEdge const & e1, GNGEdge const & e2)
{
  if(e2.m_node1 == 0 || e1.m_node1 == 0 || 
     e2.m_node2 == 0 || e1.m_node2 == 0)
     return false;
     
  bool result = e2.m_age == e1.m_age;
  result = result && e2.m_node1 == e1.m_node1;
  result = result && e2.m_node2 == e1.m_node2;
  
  return result;
}


//---------
bool operator< (GNGEdge const & e1, GNGEdge const & e2)
{ 
  return e1.m_age < e2.m_age; 
}


