/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __GNGEDGE_H
#define __GNGEDGE_H

class GNGNode;

/**
 *  Represents an UNDIRECTED edge between two nodes.
 *  When an edge is created it is given two nodes that it will
 *  register with.
 *  When it is deleted it will again notify the relevant nodes.
 */
class GNGEdge
{
public:
  /**
   * Constructs a new GNGEdge given 2 nodes.
   * @param h1 the first node
   * @param h2 the second node
   */
  GNGEdge(GNGNode * h1, GNGNode * h2);

  /**
   */
  virtual ~GNGEdge();
  
  /**
   * Given a node, retreives the other node of the edge.
   * @param node the node at one end of the edge.
   * @return the node at the other end of the edge.
   */
  GNGNode const & GetOtherNode(GNGNode const & node) const; // throws Exception

  /**
   * Given a node, retreives the other node of the edge.
   * @param node the node at one end of the edge.
   * @return the node at the other end of the edge.
   */
  GNGNode & GetOtherNode(GNGNode const & node); // throws Exception

  /**
   * Returns the first node of the edge.
   * @return the first node of the edge.
   */
  GNGNode & GetNode1() const; 

  /**
   * Returns the second node of the edge.
   * @return the second node of the edge.
   */
  GNGNode & GetNode2() const; 

  /**
   * Returns the current age of the edge.
   * @return the age of the edge.
   */
  double GetAge();

  /**
   * Sets the current age of the edge.
   * @param d the new age.
   */
  void SetAge(double d);


  friend bool operator== (GNGEdge const & e1, GNGEdge const & e2);
  friend bool operator< (GNGEdge const & e1, GNGEdge const & e2);

protected:
  double    m_age;
  GNGNode * m_node1;
  GNGNode * m_node2;  
};


#endif
