/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Exception.hpp>
#include <Vector.hpp>

#include "GNGNode.hpp"
#include "GNGEdge.hpp"
#include "EdgeContainer.hpp"

//------------
GNGNode::GNGNode()
{
  m_accumulatedError = 0.0;
  m_utility = 0.0;
}


//------------
GNGNode::~GNGNode()
{
}


//------------
void GNGNode::SetReferenceVector(Vector const & referenceVector)
{
  m_referenceVector = referenceVector;
}

//------------
Vector & GNGNode::GetReferenceVector()
{
  return m_referenceVector;
}

//------------
Vector const & GNGNode::GetReferenceVector() const
{
  return m_referenceVector;
}

//------------
void GNGNode::SetAccumulatedError(double d)
{
  m_accumulatedError = d;
}

//------------
double GNGNode::GetAccumulatedError() const
{
  return m_accumulatedError;
}

//------------
void GNGNode::SetUtility(double d)
{
  m_utility = d;
}

//------------
double GNGNode::GetUtility() const
{
  return m_utility;
}

//------------
void GNGNode::AddEdge(GNGEdge & edge)
{
  m_edges.push_back(&edge);
}

//------------
void GNGNode::RemoveEdge(GNGEdge const & edge)
{
  EdgeContainer::iterator it = m_edges.begin();
  for(; it != m_edges.end(); it++)
  {
    GNGEdge const & e = *(*it);
    if( e == edge )
    {
      // deallocation is handled by the GNGContainer.
      //  we just remove the reference in this node.
      m_edges.erase(it);
      return;
    }
  }
}


//------------
GNGEdge * GNGNode::GetEdgeTo(GNGNode const & node)
{
  // find the edge that connects (this) to (node)
  EdgeContainer::const_iterator it = m_edges.begin();
  for(; it != m_edges.end(); it++)
  {
    GNGEdge * e = (*it);
    if( &(e->GetOtherNode(*this)) == &node )
    {
      return e;
    }
  }

  return 0;
}


//------------
EdgeContainer const & GNGNode::GetEdges() const
{
  return m_edges;
}

//------------
EdgeContainer & GNGNode::GetEdges()
{
  return m_edges;
}

