/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __GNGUALGORITHM_H
#define __GNGUALGORITHM_H

#include "GNGAlgorithm.hpp"

/**
 *  Encapsulates the GNG-U algorithm descibed by Fritzke.
 *  It is directly derived from GNGAlgorithm.
 */
class GNGUAlgorithm : public GNGAlgorithm
{
public:
  /**
   * Constructs a GNG Algorithm object.
   */
  GNGUAlgorithm(unsigned int MSEBackLog) : GNGAlgorithm(MSEBackLog) {};
  
  /**
   * Iterates through the algorithm one time.
   * @param input the Input vector to use in this iteration
   */ 
  virtual void Iterate(Vector const & input);

  /**
   * Sets the Utility parameter of the GNG-U Algorithm
   * @param d the Utility parameter.
   */
	void SetParamUtility(double d) { m_paramUtility = d; }
	
  /**
   * Sets the Utility Decay parameter of the GNG-U Algorithm
   * According to the papers by Fritzke, this decay is the same as for the error.
   * @param d the Utility Decay parameter.
   */
	void SetParamUtilityDecay(double d) { m_paramUtilityDecay = d; }

protected:
  /**
   * Removes nodes that have no edges.
   */
  virtual void PurgeNodes();

  /**
   * Updates the Utility of node s
   */
  virtual void UpdateUtility(GNGNode & s, GNGNode & t);

  /**
   * Decreases the Utility of all nodes
   */
	virtual void DecreaseNodeUtiliy();

protected:
	double m_paramUtility;
  double m_paramUtilityDecay;
};


#endif
