/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __IGNGCONTAINER_H
#define __IGNGCONTAINER_H

#include "NodeContainer.hpp"
#include "EdgeContainer.hpp"

class GNGNodeFactory;

/**
 * An Interface that defines a GNGContainer.
 * IGNGContainer implementations are used by IGNGNodeInserter implementations and
 * IGNGStopCriteria implementations.
 */
class IGNGContainer
{
public:
  /**
   * Sets the factory used to create Nodes.
   * @param the node factory
   */
  virtual void SetNodeFactory(GNGNodeFactory * f) = 0;

  /**
   * Returns a reference to the Edges.
   * @return an EdgeContainer reference.
   */
  virtual EdgeContainer & GetEdges() = 0;

  /**
   * Returns a reference to the Nodes.
   * @return an NodeContainer reference.
   */
  virtual NodeContainer & GetNodes() = 0;

 /**
   * Deletes the edge at i from the internal container.
   * @param it the iterator positition of the edge to remove.
   */
  virtual void DeleteEdge(EdgeContainer::iterator & i) = 0;
  
  /**
   * Deletes a given GNGEdge from the container.
   * @param e The GNGEdge to delete from the container.
   */
  virtual void DeleteEdge(GNGEdge * e) = 0;

  /**
   * A Factory method for creating a GNGEdge between the node n1 and n2.
   * @param n1 The GNGNode at one end of the new GNGEdge
   * @param n2 The GNGNode at the other end of the new GNGEdge
   * @return The newly created GNGEdge.
   */
  virtual GNGEdge * CreateEdge(GNGNode * n1, GNGNode * n2) = 0;

  /**
   * A Factory method for creating a new GNGNode.
   * @return The newly created GNGNode.
   */
  virtual GNGNode * CreateNode() = 0;
  
 /**
   * Deletes the node at i from the internal container.
   * @param it the iterator positition of the node to remove.
   */
  virtual void DeleteNode(NodeContainer::iterator & i) = 0;
};


#endif
