/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __IGNGNODEINSERTER_H
#define __IGNGNODEINSERTER_H

class IGNGContainer;

/**
 *  An Interface that defines how GNGAlgorithm uses an IGNGNodeInserter implementation.
 *  GNGAlgorithm uses an IGNGNodeInserter implementation when insertions of new nodes are made.
 *  Implementations define how and where insertions of new nodes are made. 
 *  The insertions are made into the supplied IGNGContainer implementation.
 */
class IGNGNodeInserter 
{
public:
  /**
   * @return true if the insertion critera is met, false otherwise.
   */
  virtual bool IsInsertCriteriaMet() = 0;
  
  /**
   * This method is called to insert a new node.
   */ 
  virtual void InsertNewNode() = 0;
  
   /**
   * This method is used to register a GNGContainer object with the IGNGNodeInserter implementation.
   * It is called by GNGAlgorithm when a IGNGNodeInserter implementation is 
   * registered with GNGAlgorithm.
   * @param c An IGNGContainer object reference.
   */
 virtual void SetGNGContainer(IGNGContainer * c) = 0;
};


#endif


