/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Exception.hpp>
#include <Vector.hpp>
#include <Random.hpp>

#include "SGNGAlgorithm.hpp"
#include "NodeContainer.hpp"
#include "EdgeContainer.hpp"
#include "GNGEdge.hpp"
#include "GNGNode.hpp"
#include "IGNGContainer.hpp"
#include "IGNGNodeInserter.hpp"
#include "IGNGNodeMovement.hpp"

#include <RBF/RBFNode.hpp>

#include <cstdlib>
#include <iostream>

using namespace std;

//-----------
void SGNGAlgorithm::PurgeEdges()
{
  EdgeContainer & edges = m_gngContainer->GetEdges();
  EdgeContainer::iterator it = edges.begin();

  while(it != edges.end())
  {
    GNGEdge & edge = *(*it);
    if( edge.GetAge() > m_paramAgeMax )
		{
			GNGNode & n1 = edge.GetNode1();
			GNGNode & n2 = edge.GetNode2();
			m_gngContainer->DeleteEdge(it);

			// update the widths of the nodes that used to have an edge.
      ((RBFNode &)n1).SetWidth( CalculateMeanDistance(n1) );
      ((RBFNode &)n2).SetWidth( CalculateMeanDistance(n2) );

		}
    else
      it++;
  }
  
}

//-----------
void SGNGAlgorithm::MoveWinner(GNGNode & s)
{ 
  Vector const & currentVector = s.GetReferenceVector();
  Vector const & moveVector = m_winnerMovement->GetNewReferenceVector(m_input, currentVector, m_paramMoveWinner);
  s.SetReferenceVector( moveVector );

  // note that this is in no way the optimal method of calculating 
  // the mean distance. It is the naive way. 
  ((RBFNode &)s).SetWidth( CalculateMeanDistance(s) );
}

//-----------
void SGNGAlgorithm::MoveNeighbor(GNGNode & s)
{
  Vector const & currentVector = s.GetReferenceVector();
  Vector const & moveVector = m_neighborMovement->GetNewReferenceVector(m_input, currentVector, m_paramMoveNeighbor);
  s.SetReferenceVector( moveVector );

  // note that this is in no way the optimal method of calculating 
  // the mean distance. It is the naive way. 
  ((RBFNode &)s).SetWidth( CalculateMeanDistance(s) );

}

//-----------
double SGNGAlgorithm::CalculateMeanDistance(GNGNode & s)
{
  double mean = 0.0;
  double num = 0.0;
  
  Vector & v = s.GetReferenceVector();
  EdgeContainer const & edges = s.GetEdges();
  EdgeContainer::const_iterator it = edges.begin();
  for(; it != edges.end(); it++)
  {
    num++;
    GNGNode & node = (*it)->GetOtherNode(s);
    Vector & v2 = node.GetReferenceVector();
    mean += fabs(v.Distance(v2));
  }    
			
  return (mean/num);
}

