/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __SGNGALGORITHM_H
#define __SGNGALGORITHM_H

#include "GNGAlgorithm.hpp"
#include <GNGCore/GNGNode.hpp>
/**
 *  Encapsulates the clustering part of the SGNG algorithm descibed by Fritzke.
 *  It is directly derived from GNGAlgorithm.
 */
class SGNGAlgorithm : public GNGAlgorithm
{
public:
  /**
   * Constructs a GNG Algorithm object.
   */
  SGNGAlgorithm(unsigned int MSEBackLog) : GNGAlgorithm(MSEBackLog) {};
  
  /**
   * Sets the squared error received from the RBF network
   * @param d the squared error.
   */
  void SetSquaredError(double d) { m_squaredError = d; }
	
protected:
  /**
   * Updates the local error of a node with the current squared error from the RBF net.
   * @param s the node that will be updated.
   */
  virtual void UpdateError(GNGNode & s) 
  { 
  	s.SetAccumulatedError( s.GetAccumulatedError() + m_squaredError ); 
  }
	
  /**
   * Removes edges that are too old. and updates the widths of the nodes that shared an edges.
   */
  virtual void PurgeEdges();

  /**
   * Moves the winner node s towards the current input vector by a factor of paramMoveWinner.
   * Since the nodes have been moved, the mean distance to all neighbours must be updated. 
   * To change the behaviour of winner movment, overload this method.
   * @param s the node to move
   */
  virtual void MoveWinner(GNGNode & s);
  
  /**
   * Moves a neighbor node s towards the input vector by a factor of paramMoveNeighbor.
   * To change the behaviour of neighbor movment, overload this method.
   * Since the nodes have been moved, the mean distance to all neighbours must be updated. 
   * @param s the node to move
   */
  virtual void MoveNeighbor(GNGNode & s);

  /**
   * Calculated the mean distance to all neigbours of node s.
   * @param s a node
   * @return the mean distance to the neighbours of s.
   */
  virtual double CalculateMeanDistance(GNGNode & s);
  
  
protected:
   double m_squaredError;
};


#endif
