/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Control.hpp>
#include <GNGCore/EdgeContainer.hpp>
#include <GNGCore/NodeContainer.hpp>
#include <GNGCore/GNGEdge.hpp>
#include <GNGCore/GNGNode.hpp>
#include <GNGCore/IGNGContainer.hpp>

#include "GNGGLUTRepresentation.hpp"
#include "IGLInputRepresentation.hpp"

#include "GNGRepresentation.hpp"
#include <fstream>
#include <iostream>

#ifdef win32
#include "wtypes.h"
#include "winbase.h"
#endif

#include <GL/glut.h>
#include <sstream>
#include <iostream>

using namespace std;

//-----------------------------------------------------------------------------

IGNGContainer * GNGGLUTRepresentation::m_gngContainer = 0;
Control * GNGGLUTRepresentation::m_control = 0;
IGLInputRepresentation * GNGGLUTRepresentation::m_inputRepresentation = 0;

unsigned int GNGGLUTRepresentation::m_counter = 1;
unsigned int GNGGLUTRepresentation::m_representationUpdate = 0;

double GNGGLUTRepresentation::m_xMin = -1.1;
double GNGGLUTRepresentation::m_xMax = 1.1;
double GNGGLUTRepresentation::m_yMin = -1.1;
double GNGGLUTRepresentation::m_yMax = 1.1;
bool   GNGGLUTRepresentation::m_paused = true;
bool   GNGGLUTRepresentation::m_stepOnce = false;
bool   GNGGLUTRepresentation::m_stepSequence = false;


void GNGGLUTRepresentation::SetInputRepresentation(IGLInputRepresentation * i) 
{ 
  m_inputRepresentation = i; 
  m_xMax = i->GetDisplayXMax() + 0.1;
  m_yMax = i->GetDisplayYMax() + 0.1;
  m_xMin = i->GetDisplayXMin() - 1.1;   // extra space allocated for text.
  m_yMin = i->GetDisplayYMin() - 0.1;

  glMatrixMode(GL_PROJECTION);
  glOrtho(m_xMin, m_xMax, m_yMin, m_yMax, -500.0, 500.0);	
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  double width = 660;//950;
  double height = width * (m_yMax - m_yMin)/(m_xMax - m_xMin);
  glutReshapeWindow((int)width, (int)height); 
}


//----------
GNGGLUTRepresentation::GNGGLUTRepresentation(char ** argv, int argc)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	
	glutCreateWindow("GNG OpenGL Representation");
	
	glutDisplayFunc(CallBackDisplayFunc);
	glutReshapeFunc(CallBackReshapeFunc);
	glutIdleFunc(CallBackIdleFunc);
		
	glPointSize(5.0);
	glutMouseFunc(CallBackMouseFunc);

    glClearColor(1.0, 1.0, 1.0, 1.0);
}

//----------
void GNGGLUTRepresentation::Run()
{
	if(m_gngContainer == 0 || m_control == 0 || m_inputRepresentation == 0)
		throw Exception("GNGGLUTRepresentation not initialized properly, components are null.");
	
	glutMainLoop();
}


//----------
void GNGGLUTRepresentation::CallBackMouseFunc(int mouse, int state, int x, int y)
{
	if(state==GLUT_DOWN) 
	{
		if(mouse==GLUT_LEFT_BUTTON)
		{
			m_paused = !m_paused;
			if(m_paused)
			  m_control->Snapshot();
		}
		else 
		if(mouse==GLUT_RIGHT_BUTTON)
		{
			m_stepOnce = true;
		}
		else
		if(mouse==GLUT_MIDDLE_BUTTON)
		{
			m_stepSequence = true;			
		}
	}
}


//----------
void GNGGLUTRepresentation::CallBackIdleFunc(void)
{
	if(m_paused && !m_stepOnce && !m_stepSequence)
	{	
		return;
	}
	else
	{
		if( m_control->Iterate() )
		{
			m_counter = m_control->GetIteration();

            // step once
            if((m_stepOnce && m_paused))
            {
				GNGGLUTRepresentation::OutputRepresentation();
            }
			else // step sequence
		    if((m_counter % m_representationUpdate) == 0 && m_paused)
			{
			    m_stepSequence = false;
			    GNGGLUTRepresentation::OutputRepresentation();

			    m_control->Snapshot();
			}
            else // normal
			if((m_counter % m_representationUpdate) == 0)
			{
				GNGGLUTRepresentation::OutputRepresentation();
			}
		}
		m_stepOnce = false;
	}
}



//----------
void GNGGLUTRepresentation::CallBackDisplayFunc(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	
	Output();
	
	glutSwapBuffers();
}

//----------
void GNGGLUTRepresentation::CallBackReshapeFunc(int w, int h)
{  
	glViewport(0, 0, w, h); 
	CallBackDisplayFunc();
}


//----------
void GNGGLUTRepresentation::OutputRepresentation()
{
	glutPostRedisplay();
}



//----------
// Ouput creates all the nececary GL calls to produce
//  a representation of the state of the GNGAlgorithm
//
void GNGGLUTRepresentation::Output()
{ 
	// ouput InputRepresentation
    glColor4f(0.8, 0.8, 0.8, 1.0);
	m_inputRepresentation->GLOutputDistribution();

    glColor4f(0.0, 0.0, 0.0, 1.0);
	m_inputRepresentation->GLOutputSignal();

	OutputEdges();
	OutputNodes();
	OutputText();
}

void GNGGLUTRepresentation::OutputEdges()
{
	IGNGContainer & c = *m_gngContainer;
	
	EdgeContainer const & edges = c.GetEdges();
	EdgeContainer::const_iterator it = edges.begin();
    
	glBegin(GL_LINES);
	{
		glColor4f(0.0, 0.0, 1.0, 1.0);
		for(;it != edges.end(); it++)
		{			
			GNGEdge const & edge = *(*it);
			OutputEdge(edge);
		}
	}    
	glEnd();
}	


void GNGGLUTRepresentation::OutputNodes()
{
	IGNGContainer & c = *m_gngContainer;
	
	NodeContainer const & nodes = c.GetNodes();
	NodeContainer::const_iterator it = nodes.begin();
    
	glBegin(GL_POINTS);
	{			
		glColor4f(1.0, 0.0, 0.0, 1.0);
		for(;it != nodes.end(); it++)
		{
			GNGNode const & node = *(*it);
			OutputNode(node);      
		}
		it--;
		// latest inserted node.
		glColor4f(0.0, 1.0, 0.0, 1.0);
		OutputNode(*(*it));
	}    
	glEnd();
}

void GNGGLUTRepresentation::OutputText()
{    
	// # iterations
	{      
		glColor4f(0.0, 0.0, 0.0, 1.0);
		glPushMatrix();
		glTranslatef(m_xMin + 0.05 , m_yMax - 0.2 ,0.0);
		glScalef(0.0004,0.0004,0.0004);
		
		stringstream ss;
		ss << "Iteration: #" << m_counter;
		while(!!ss)
			glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, ss.get());      
		glPopMatrix();
	}
    
	// # nodes
	{      
		glColor4f(0.0, 0.0, 0.0, 1.0);
		glPushMatrix();
		glTranslatef(m_xMin + 0.05 , m_yMax - 0.3 ,0.0);
		glScalef(0.0004,0.0004,0.0004);
		
		stringstream ss;
		ss << "Nodes: #" << m_gngContainer->GetNodes().size();
		while(!!ss)
			glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, ss.get());      
		glPopMatrix();
	}
}


//----------
void GNGGLUTRepresentation::OutputNode(GNGNode const & node)
{
	Vector const & v = node.GetReferenceVector();
	if(v.size() >= 2)
	  glVertex2d(v[0], v[1]);
    else
    if(v.size() == 1)
	  glVertex2d(v[0], 0.0);
}


//----------
void GNGGLUTRepresentation::OutputEdge(GNGEdge const & edge)
{
	Vector const & v1 = edge.GetNode1().GetReferenceVector();
	Vector const & v2 = edge.GetNode2().GetReferenceVector();
	
	if(v1.size() >= 2)
	{
      glVertex2d(v1[0], v1[1]);
	  glVertex2d(v2[0], v2[1]);
	}
	else
	if(v1.size() == 1)
	{
      glVertex2d(v1[0], 0.0);
	  glVertex2d(v2[0], 0.0);
	}
	
}
