/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __GNGGLUTREPRESENTATION_HH
#define __GNGGLUTREPRESENTATION_HH

#include <GNGCore/GNGAlgorithm.hpp>
#include <GNGCore/EdgeContainer.hpp>
#include <GNGCore/NodeContainer.hpp>
#include <GL/glut.h>
#include "IGLInputRepresentation.hpp"

class IGNGContainer;
class GNGNode;
class GNGEdge;
class Control;

/**
* Provides a means for ouputing a representation of the GNG net to an openGL window. 
* GNGGLUTRepresentation uses GLUT to open a window. It uses an IGLInputRepresentation 
* to display the input space and a GNGAlgorithm object to display the state of the 
* GNG net.
* Since GLUT has its own main loop, you must register an IdleObject with GNGGLUTRepresentation
* which is used in its main loop when it is not repainting the graphics. The IdleObject contains
* the code that should be run in the main loop.
*/
class GNGGLUTRepresentation
{
public:
	GNGGLUTRepresentation(char ** argv, int argc);
	virtual ~GNGGLUTRepresentation() 
	{ 
		delete m_inputRepresentation; 
	}

	static void OutputRepresentation();
	void Run();

	void SetGNGContainer(IGNGContainer * c)                 { m_gngContainer = c; }
	void SetControlObject(Control * i)                      { m_control = i; }
	void SetUpdateInterval(unsigned int i)                  { m_representationUpdate = i; }
	void SetInputRepresentation(IGLInputRepresentation * i);

private:
	static void CallBackMouseFunc(int mouse, int state, int x, int y);
	static void CallBackDisplayFunc(void);
	static void CallBackReshapeFunc(int w, int h);
	static void CallBackIdleFunc(void);

	static void OutputEdges();
	static void OutputNodes();
	static void OutputText();
	static void OutputNode(GNGNode const & node);
	static void OutputEdge(GNGEdge const & edge);
	static void Output();

private:
	static unsigned int              m_counter;
	static Control *                 m_control;
	static unsigned int              m_representationUpdate;
	static IGLInputRepresentation *  m_inputRepresentation;
	static IGNGContainer *           m_gngContainer;	
	static double                    m_xMin;
	static double                    m_xMax;
	static double                    m_yMin;
	static double                    m_yMax;
	static bool                      m_paused;
	static bool                      m_stepOnce;
	static bool                      m_stepSequence;
};

#endif



