/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <GNGCore/EdgeContainer.hpp>
#include <GNGCore/NodeContainer.hpp>
#include <GNGCore/GNGEdge.hpp>
#include <GNGCore/GNGNode.hpp>
#include <GNGCore/IGNGContainer.hpp>

#include <RBF/RBFNode.hpp>

#include "GNGRepresentation.hpp"

#include <iostream>
using namespace std;

//----------
// Write all nodes to std out, pair them if they have an edge.
//  
//
void GNGRepresentation::OutputRepresentationPaired(IGNGContainer & c)
{
  NodeContainer const & nodes = c.GetNodes();
  NodeContainer::const_iterator it = nodes.begin();

  for(;it != nodes.end(); it++)
  {
    GNGNode * node = (*it);
    EdgeContainer const & edges = node->GetEdges();

    if( edges.empty() )
    {
      OutputNode(node);
      m_ostream << endl;
    }
    else
      OutputNodeEdges(node);
  }
}

void GNGRepresentation::OutputRepresentation(IGNGContainer & c)
{
  NodeContainer const & nodes = c.GetNodes();
  NodeContainer::const_iterator it = nodes.begin();
  NodeContainer::const_iterator itForward = nodes.begin();
  itForward++;
  
  m_ostream << "set sample 10000" << endl << "set xrange [0:7]" << endl;
  m_ostream << "plot \\" << endl;
  for(;itForward != nodes.end(); itForward++)
  {
    OutputNode( *it );
    m_ostream << " notitle , \\" << endl;
    it++;
  }
  
  if(nodes.end() != it)
    OutputNode( *it );
  
  m_ostream << " notitle " << endl;
}

//----------
void GNGRepresentation::OutputNode(GNGNode const * node)
{
  Vector const & v = node->GetReferenceVector();
  
	//for(unsigned int i=0; i<v.size()-1; i++)
  //  m_ostream << v[i] << ", ";
  //m_ostream << v[v.size()-1] << " " << n->GetWidth();

	RBFNode const * n = (RBFNode const *)node;
	m_ostream << " exp( -(x-" << v[0] << ")**2 / ( 2*" << n->GetWidth() << "**2) )";
    //m_ostream << v << " " << n->GetWidth() << endl;
}


//----------
void GNGRepresentation::OutputNodeEdges(GNGNode const * node)
{
  EdgeContainer const & edges = node->GetEdges();

  EdgeContainer::const_iterator it = edges.begin();
  for(; it != edges.end(); it++)
  {
    GNGEdge const & edge = *(*it);
    OutputNode(node);
		GNGNode const * o = &edge.GetOtherNode(*node);
    OutputNode(o);
    m_ostream << endl;
  }
  
}
