/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __GNGREPRESENTATION_HH
#define __GNGREPRESENTATION_HH

#include "../GNGCore/EdgeContainer.hpp"
#include "../GNGCore/NodeContainer.hpp"

#include <iostream>

class IGNGContainer;
class GNGNode;
class GNGEdge;

/**
 * Provides a means for ouputing a representation of
 * a IGNGContainer implementation to an outputstream.
 */
class GNGRepresentation
{
public:
  /**
   * Initialize a new GNGRepresentation with an outputstream.
   * @param o the outputstream to which the representation is output.
   */
  GNGRepresentation(std::ostream & o) : m_ostream(o) {}
  virtual ~GNGRepresentation() {}

  /**
   * Outputs a representation of the provided IGNGContainer. 
	 * nodes are paired if they have an edge.
   * @param c an IGNGContainer 
   */
  virtual void OutputRepresentationPaired(IGNGContainer & c);	

  /**
   * Outputs a representation of the provided IGNGContainer. 
   * @param c an IGNGContainer 
   */
	virtual void OutputRepresentation(IGNGContainer & c);

private:
  /**
   * Outputs a GNGNode.
   * @param node a GNGNode to output.
   */
  void OutputNode(GNGNode const * node);

  /**
   * Outputs all the GNGEdges that node has.
   * @param node the GNGNode to get the edges from.
   */
  void OutputNodeEdges(GNGNode const * node);

private:
  std::ostream &  m_ostream;

};

#endif



