/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __IINPUTGENERATOR_H
#define __IINPUTGENERATOR_H

#include <Vector.hpp>

/**
 *  An Interface for Input gerenators.
 */

class IInputGenerator 
{  
public:

  enum eGeneratorType {FunctionGenerator, ClassesGenerator};
  typedef enum eGeneratorType GeneratorType;
  
  /**
   * Generates an Input Vector.
   * @return The generated Input Vector.
   */
  virtual Vector GetInput() = 0;

  /**
   * Returns the output corresponding to the latest generated input.
   * Outputs are generally coded in a one-out-of-k fashion.
   * Must be implemented, but is not used by GNG or GNG-U.
   * @return the output corresponding to the latest generated input.
   */
  virtual Vector GetOutput()
  {
    return m_output;
  }

  /**
   * Returns a reference to the Vector most recently generated by a call to GetInput().
   * @return The Input Vector.
   */
  virtual Vector const & GetLastInput() const = 0;

  /**
   * Returns the dimensionality if the input vectors that are generated.
   * @return The dimension of the inputs that are generated.
   */
  virtual unsigned int GetDimension() const = 0;
  
  /**
   * Returns the type of generator
   * @return the type of gererator, either a function generator or a classes generator 
   */
  virtual GeneratorType GetGeneratorType() const = 0;

protected:
  Vector m_output;

};


#endif

