/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __INPUTGENERATORDEFAULT_H
#define __INPUTGENERATORDEFAULT_H

#include <Random.hpp>
#include "IInputGenerator.hpp"
#include <GNGRepresentation/IGLInputRepresentation.hpp>

#include <cstdlib>
#include <GL/glut.h>

/**
 * Generates m_dimensional signals from a hypercubical distribution.
 */
class InputGeneratorDefault : public IInputGenerator
{ 
public:
  /**
   * @param dimension - The dimension of the input to be generated.
   */
  InputGeneratorDefault(unsigned int dimension)
  {
    m_dimension = dimension;
    m_min = -1.0;
    m_max = 1.0;

    for(unsigned int i=0; i< dimension; i++)
        m_vector.push_back(0.0);
        
    m_output.push_back(1.0); // only one class.
  }
  
  /**
   * @return An m_dimensional vector.
   */
  virtual Vector GetInput()
  {
    for(unsigned int i=0; i<m_dimension; i++)
      m_vector[i] = Random::Randomize();
    
    return m_vector;
  }
  
  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_vector; }  

  /**
   * @return The dimension, which is deciced at construction time.
   */
  virtual unsigned int GetDimension() const { return m_dimension; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return ClassesGenerator; }
  
protected:
  unsigned int m_dimension;
  double m_min;
  double m_max;
  Vector m_vector;
};


/**
 * An openGL representation. A class that represents the DefaultInput Generators domain.
 */ 
class InputRepresentationDefault : public IGLInputRepresentation
{
public:
  InputRepresentationDefault(InputGeneratorDefault * i) : m_inputGenerator(*i) {}
  virtual ~InputRepresentationDefault() {}
  
  virtual void GLOutputDistribution() const
  {
    glRectd(1.0,1.0,-1.0,-1.0);  
  }
  
  virtual void GLOutputSignal() const 
  {    
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
    
  }
  
private:
  InputGeneratorDefault & m_inputGenerator;
};

#endif

