/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __INUTGENERATORTWOCLASSES_H
#define __INUTGENERATORTWOCLASSES_H

#include <Random.hpp>
#include "IInputGenerator.hpp"

class InputGeneratorTwoClasses : public IInputGenerator
{
public: 
  InputGeneratorTwoClasses(double width, double height)
  {
    m_input.push_back(0.0);
    m_input.push_back(0.0);
    m_output.push_back(0.0);
    m_output.push_back(0.0);
    m_width = width;
    m_height = height;
    m_dimension = 2;
  }
  
  virtual Vector GetOutput()
  {
    return m_output;
  }

  /**
   * @return a 2-dim vector representing the TwoClasses boolean funtion.
   */
  virtual Vector GetInput()
  {
    double x = Random::Randomize();
    if(x < 0.0)
    {
      m_input[0] = -1.0 - x * m_width;
      m_output[0] = 0.0;
      m_output[1] = 1.0;
    }
    else
    {
      m_input[0] = 1.0 - x * m_width;
      m_output[0] = 1.0;
      m_output[1] = 0.0;
    }

    m_input[1] = Random::Randomize() * m_height;
    
    return m_input;
  }
  
  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_input; }  

  /**
   * @return The dimension, which is deciced at construction time.
   */
  virtual unsigned int GetDimension() const { return m_dimension; }
  
  virtual double GetWidth() { return m_width; }
  virtual double GetHeight() { return m_height; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return ClassesGenerator; }
  
protected:
  unsigned int m_dimension;
  double m_width;
  double m_height;
  Vector m_input;  
};


/**
 * An openGL representation. A class that represents the InputGeneratorTwoClasses domain.
 */ 
class InputRepresentationTwoClasses : public IGLInputRepresentation
{
public:
  InputRepresentationTwoClasses(InputGeneratorTwoClasses * i) : m_inputGenerator(*i) {}
	virtual ~InputRepresentationTwoClasses() {}

  virtual void GLOutputDistribution() const
  {
    double width = m_inputGenerator.GetWidth();
    double height = m_inputGenerator.GetHeight();

    double yMin = -height;
    double yMax = height;
    double xMin = -1.0 + width;
    double xMax = 1.0 - width;

    glColor4f(0.6, 0.6, 0.6, 1.0);
    glRectd(-1.0,yMin, xMin,yMax);  
    glColor4f(0.8, 0.8, 0.8, 1.0);
    glRectd(1.0,yMin, xMax,yMax);  
  }
  
  virtual void GLOutputSignal() const 
  {    
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
  }
  
private:
  InputGeneratorTwoClasses & m_inputGenerator;
};


#endif
