/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __INPUTGENERATORXOR_H
#define __INPUTGENERATORXOR_H

#include "IInputGenerator.hpp"

/**
* A generator that generates inputs and corresponding 
* outputs to the XOR boolean funciton.
*/
class InputGeneratorXOR : public IInputGenerator
{
public: 
  InputGeneratorXOR()
  {
    m_dimension = 2;
    m_min = 0.0;
    m_max = 1.0;

    m_input.push_back(0.0);
    m_input.push_back(0.0);
    m_output.push_back(0.0);
  }

  /**
   * @return a 2-dim vector representing the xor boolean funtion.
   */
  virtual Vector GetInput()
  {    
    if(Random::RandomizePositive() < 0.5)
      m_input[0] = 0.0;
    else 
      m_input[0] = 1.0;

    if(Random::RandomizePositive() < 0.5)
      m_input[1] = 0.0;
    else 
      m_input[1] = 1.0;
    
    if(m_input[0] + m_input[1] == 1.0) // xor
      m_output[0] = 1.0;
    else 
      m_output[0] = 0.0;

    return m_input;
  }
    
  /**
   * @return the 1-dim class for the current input, 0 or 1
   */
  virtual Vector GetOutput()
  {
    return m_output;
  }

  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_input; }  

  /**
   * @return The dimension, which is deciced at construction time.
   */
  virtual unsigned int GetDimension() const { return m_dimension; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return FunctionGenerator; }
  
protected:
  unsigned int m_dimension;
  double m_min;
  double m_max;
  Vector m_input;  
};


/**
 * An openGL representation. A class that represents the InputGeneratorXOR domain.
 */ 
class InputRepresentationXOR : public IGLInputRepresentation
{
public:
  InputRepresentationXOR(InputGeneratorXOR * i) : m_inputGenerator(*i) {}
	~InputRepresentationXOR() {}

  virtual void GLOutputDistribution() const
  {
    glBegin(GL_POINTS);
    {			
      glColor4f(0.6, 0.6, 0.6, 1.0);
      glVertex2d(0, 1);
      glVertex2d(1, 0);
      glColor4f(0.8, 0.8, 0.8, 1.0);
      glVertex2d(1, 1);
      glVertex2d(0, 0);
    }    
    glEnd();
  }
  
  virtual void GLOutputSignal() const 
  {    
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
  }
  
private:
  InputGeneratorXOR & m_inputGenerator;
};


#endif
