/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFHIDDENLAYER_H
#define __RBFHIDDENLAYER_H

#include <Vector.hpp>
#include <GNGComponents/DefaultGNGContainer.hpp>

#include "RBFNode.hpp"

#include "IWeightLayerToHidden.hpp"


/**
* The Hidden layer in a SGNG-RBF Network.
*/
class RBFHiddenLayer : public DefaultGNGContainer
{
public:
	RBFHiddenLayer(IWeightLayerToHidden * w)
	{
		m_weightLayer = w;
	}

	/**
	* Applies the input vector and stores the output signal in the weightlayer
	* @param input the input signal.
	*/
	virtual void ApplyInput(Vector const & input)
	{
		// iterate through all nodes and calc their Activation
		NodeContainer::iterator it = m_nodes.begin();
		for(; it != m_nodes.end(); it++)
		{
			RBFNode * n = (RBFNode *)(*it);
			m_weightLayer->SetSignal(n, n->Activation(input));
		}
	}

	/**
	 * OVERLOADED from DefaultGNGContainer
	 * Modified node creation, so that it also create a Weights object
	 * that connects it to all the output nodes.
	 */
	GNGNode * CreateNode()
	{  
		GNGNode * node = m_nodeFactory->NewNode();    
		m_nodes.push_back(node);

		// Create a Weights Object in WeightLayer, associate it with n.
		RBFNode * n = (RBFNode *)node;
		m_weightLayer->CreateWeightsObject(n);

		return node;
	}  

	/**
	 * OVERLOADED from DefaultGNGContainer
	 * Modified node deletion, so that it also deletes the Weights object
	 * that connects it to all the output nodes.
	 * @param i the position of the node to delete.
	 */
	void DeleteNode(NodeContainer::iterator & i)
	{
		RBFNode * n = (RBFNode *)(*i);
		m_weightLayer->DeleteWeightsObject(n);

		delete (*i);      // free memory
		m_nodes.erase(i); // remove reference.
	}

private:
	IWeightLayerToHidden *  m_weightLayer;
};


#endif
