/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFNODE_H
#define __RBFNODE_H

#include <GNGCore/GNGNode.hpp>
#include "RBFWeights.hpp"
#include <cmath>

/**
 * A Gaussian RBFNode
 */
class RBFNode : public GNGNode
{
public:
	RBFNode(double sigma) 
	{ 
		m_sigma = sigma;
	} 

	virtual double Activation(Vector const & input)
	{
		double squaredDistance = input.SquaredDistance( GetReferenceVector() );
		double ret = exp( squaredDistance / (-2.0*pow(m_sigma,2.0)) );
		return ret;
	}

	virtual void SetWidth(double sigma) 
  { 
    m_sigma = sigma;
    //m_sigma = sigma * 2.0;
    //std::cout << "\n SIGMA: " << m_sigma << "  " << std::endl;     
  }

	virtual double GetWidth() const { return m_sigma; }
private:
	double m_sigma; // the standard deviation of the Gaussian, or the width
};




class RBFNodeFactory : public GNGNodeFactory
{
public:
	RBFNodeFactory(double defaultSigma) { m_sigma = defaultSigma; }
	virtual GNGNode * NewNode() { return new RBFNode(m_sigma); }

private:
	double m_sigma;
};

#endif

