/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFOUTPUTLAYER_H
#define __RBFOUTPUTLAYER_H

#include "IWeightLayerToOutput.hpp"
#include "IActivation.hpp"

#include <Vector.hpp>
#include <iostream>


class RBFOutputLayer
{
public:
    RBFOutputLayer(IActivation * a, unsigned int nofOutputs, IWeightLayerToOutput * w)
    {
        m_weightLayer = w;
        m_activation = a;
        m_nofOutputs = nofOutputs;
		
		m_biases = Vector(m_nofOutputs);
    }

	Vector GetOutput()
	{
		Vector v(m_nofOutputs);
		for(unsigned int i=0; i<m_nofOutputs; i++)
			v[i] = m_activation->ActivationFucntion( m_weightLayer->GetWeightedSum(i) + m_biases[i] );

		return v;
	}

	unsigned int GetNofOutputs() { return m_nofOutputs; }

	void UpdateBias(Vector & etaDiff)
	{
		int b= m_biases.size();
		int a = etaDiff.size();
		m_biases += etaDiff;
	}

private:
  IActivation *           m_activation;
  unsigned int            m_nofOutputs;
  Vector                  m_biases;
  IWeightLayerToOutput *  m_weightLayer;
};

#endif

