/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef RBFWEIGHTLAYER_H
#define RBFWEIGHTLAYER_H

#include "IWeightLayerToHidden.hpp"
#include "IWeightLayerToOutput.hpp"
#include <map>


/**
* The WeightLayer consists of Weights objects, each of 
* which is associated to ONE RBFNode and one or more output nodes.
*/
class RBFWeightLayer : public IWeightLayerToHidden, public IWeightLayerToOutput
{
public:
	RBFWeightLayer(unsigned int nofOutputNodes) { m_nofOutputNodes = nofOutputNodes; }
	~RBFWeightLayer()
	{
		std::map<RBFNode *, RBFWeights *>::iterator it = m_weights.begin();

		// free some memory
		for(; it != m_weights.end(); it++)
			delete (*it).second;

	}

	virtual void CreateWeightsObject(RBFNode * n)
	{
		RBFWeights * w = new RBFWeights(m_nofOutputNodes);
		m_weights[n] = w;
	}

	virtual void DeleteWeightsObject(RBFNode * n)
	{
		delete m_weights[n];
		m_weights.erase(n);
	}

	virtual void SetSignal(RBFNode * n, double s)
	{
		m_weights[n]->SetSignal(s);
	}

	virtual double GetWeightedSum(unsigned int outputNodeIndex)
	{
		double weightedSum = 0.0;

		std::map<RBFNode *, RBFWeights *>::iterator it = m_weights.begin();
		for(; it != m_weights.end(); it++)
		{
			RBFWeights & w = *((*it).second);
			weightedSum += w.GetWeightedSignal(outputNodeIndex);
		}

		return weightedSum;
	}

	virtual void UpdateWeights(Vector & etaDiff)
	{
		std::map<RBFNode *, RBFWeights *>::iterator it = m_weights.begin();
		for(; it != m_weights.end(); it++)
			(*it).second->UpdateWeights(etaDiff);
	}

private:
	std::map<RBFNode *, RBFWeights *>  m_weights;
	unsigned int                       m_nofOutputNodes;


};

#endif








