/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFWEIGHTS_H
#define __RBFWEIGHTS_H

#include <Random.hpp>
#include <Exception.hpp>
#include <Vector.hpp>

//#include <iostream>

/**
*  A Weights object collects all the weights needed to connect ONE hidden node to ALL output nodes.
*/
class RBFWeights
{
public:
	RBFWeights(unsigned int size) 
	{ 
		m_size = size;
		m_weights = new double[size]; 
		if(m_weights == 0)
			throw Exception("out of memory");

		for(unsigned int i=0; i<size; i++)
			m_weights[i] = Random::Randomize(); // between -1.0 and 1.0
	}

	~RBFWeights() { delete [] m_weights; }

	double GetWeight(unsigned int i) { return m_weights[i]; }
	void   SetWeight(unsigned int i, double w) { m_weights[i] = w; }

	void   SetSignal(double i) { m_signal = i; }
	double GetWeightedSignal(unsigned int i) { return m_weights[i] * m_signal; }  // hr kan man optimera hastighet om man sparar resultatet.

	void   UpdateWeights(Vector & etaDiff)
	{
		//std::cout << "A: " << m_signal << " " ;

		Vector::iterator it = etaDiff.begin();
		for(unsigned int i=0; it!=etaDiff.end(); it++, i++)
			m_weights[i] += (*it) * m_signal;
	}

private:
	unsigned int m_size;
	double *     m_weights;
	double       m_signal;
};


#endif
