/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#include <iostream>
#include <fstream>
#include <string>
#include "Settings.hpp"

using namespace std;

Settings::Settings(std::string filename)
{
	std::ifstream settings(filename.c_str());
	if(!settings)
	{
		std::cerr << "INI-file not found, using defaults." << std::endl;
		m_mode = "gng";
		m_dim = 2;                  // dimension of input and reference vectors
		
        m_initFromDistribution = false;
		
		m_representationUpdate = 50;// a limit for number of iterations.
		
		m_maxNodes = 200;           // Max Nodes
		m_insertEvery = 600;        // Lambda
		m_newNodePos = 0.5;         // --finns inte i javaappleten.
		m_errorDecayInsert = 0.5;   // Alfa
		
		m_ageMax = 88.0;            // Age Max
		m_moveWinner = 0.05;        // Epsilon Winner
		m_moveNeighbor = 0.0006;    // Epsilon Neighbor
		m_errorDecay = 0.0005;      // Beta
		m_utilityDecay = 0.0005;    // samma som Beta enligt
		m_utilityBound = 3.0;       // U - Utility
		m_distributionType = 1;
		m_iterationDelay = 0;       // disabled
		
		m_GNGMSEBackLog = 500;
		
        m_RBFbackLog = 500;            //MISCLASSIFICATION_BACKLOG
        m_maxMisclassification = 1; //NUM_MISCLASSIFICATIONS   this means 1 in 500
        m_classificaitonDecisionDelta = 0.0; 
        m_classificaitonDecisionLimit = 0.7; 
        m_maxSquaredError = 0.01;    // max allowed error in function approximation
        
        m_eta = 0.2;
	}
	else
	{
	 //	
     // Mode
	 //	
	    cout << "*** MODE ***" << endl;	    
		SkipComment(settings);
		settings >> m_mode;
        cerr << "Mode: " << m_mode << endl;

		SkipComment(settings);
		settings >> m_initFromDistribution;
        cerr << "Init from distribution: " << m_initFromDistribution << endl;

	 //	
     // Representation
	 //	
	    cout << "\n*** Representation ***" << endl;	    
		SkipComment(settings);
		settings >> m_representationUpdate;
        cerr << "REPRESENTATION_UPDATE: " << m_representationUpdate << endl;
		
		SkipComment(settings);
		settings >> m_iterationDelay;
        cerr << "ITERATION_DELAY_IN_MILLISECONDS: " << m_iterationDelay << endl;
						
	 //	
     // GNG settings
	 //	
	    cout << "\n*** GNG ***" << endl;
		SkipComment(settings);
		settings >> m_GNGMSEBackLog;
		cerr << "GNG MSE BackLog: " << m_GNGMSEBackLog << endl;
		
		SkipComment(settings);
		settings >> m_maxNodes;
	    cerr << "MAXNODES: " << m_maxNodes << endl;
		
		SkipComment(settings);
		settings >> m_insertEvery;
        cerr << "INSERTEVERY: " << m_insertEvery << endl;
		
		SkipComment(settings);
		settings >> m_newNodePos;
        cerr << "NEWNODEPOS: " << m_newNodePos << endl;
		
		SkipComment(settings);
		settings >> m_errorDecayInsert;
        cerr << "ERRORDECAYINSERT: " << m_errorDecayInsert << endl;
		
		SkipComment(settings);
		settings >> m_ageMax;
        cerr << "AGEMAX: " << m_ageMax << endl;
		
		SkipComment(settings);
		settings >> m_moveWinner;
        cerr << "MOVEWINNER: " << m_moveWinner << endl;
		
		SkipComment(settings);
		settings >> m_moveNeighbor;
        cerr << "MOVENEIGHBOR: " << m_moveNeighbor << endl;
		
		SkipComment(settings);
		settings >> m_errorDecay;
        cerr << "ERRORDECAY: " << m_errorDecay << endl;
		
	 //	
     // GNG-U specific
	 //	
	    cout << "\n*** GNG-U ***" << endl;	    
		SkipComment(settings);
		settings >> m_utilityDecay;
	    cerr << "UTILITYDECAY: " << m_utilityDecay << endl;
		
		SkipComment(settings);
		settings >> m_utilityBound;
	    cerr << "UTILITYBOUND: " << m_utilityBound << endl;
		

	 //	
     // SGNG (GNG and RBF)
	 //	
	    cout << "\n*** SGNG ***" << endl;

		SkipComment(settings);
		settings >> m_RBFbackLog;
        cerr << "RBF_BACKLOG: " << m_RBFbackLog << endl;
		
		SkipComment(settings);
		settings >> m_maxMisclassification;
        cerr << "MAX_MISCLASSIFICATIONS: " << m_maxMisclassification << endl;

		SkipComment(settings);
		settings >> m_classificaitonDecisionDelta;
        cerr << "RBF_CLASSIFICATION_DECISION_DELTA: " << m_classificaitonDecisionDelta << endl;	

		SkipComment(settings);
		settings >> m_classificaitonDecisionLimit;
        cerr << "RBF_CLASSIFICATION_DECISION_LIMIT: " << m_classificaitonDecisionLimit << endl;	

		SkipComment(settings);
		settings >> m_maxSquaredError;
        cerr << "RBF_MAX_SQUARED_ERROR: " << m_maxSquaredError << endl;

		SkipComment(settings);
		settings >> m_eta;
        cerr << "RBF_ETA: " << m_eta << endl;

	 //	
     // Generator stuff
	 //	
	    cout << "\n*** Generators ***" << endl;	    
		SkipComment(settings);
		settings >> m_dim;
	    cerr << "DIM: " << m_dim << endl;

		//SkipComment(settings);
		//settings >> m_maxIterations;
	    //cerr << "MAXITERATIONS: " << m_maxIterations << endl;
		
		SkipComment(settings);
		settings >> m_distributionType;
	    cerr << "DISTRIBUTION_TYPE: " << m_distributionType << endl;
	}
}

void Settings::SkipComment(std::ifstream & s)
{
    bool stop = false;
    
    while(!s.eof())
    {
      char token = s.peek();
      if(token == '#')
      {
        //std::cout << "::#" << endl;
        while(s.peek() != '\n')
          s.ignore();
        s.ignore();
      }
      else
      if(token == '\n')
      {
        //std::cout << "::endl" << endl;
        s.ignore();
      }
      else
      {
        //std::cout << "::"<< token << endl;
        if(token == ' ' || token == '\t')
          s.ignore();
        else
          break;
      }      
    }
}





/*
	{
		std::string s;
		
		settings >> s;
		settings >> m_usingRBF;

		settings >> s;
		settings >> m_backLog;
		
		settings >> s;
		settings >> m_maxMisclassification;

		settings >> s;
		settings >> m_maxSquaredError;
		
		settings >> s;
		settings >> m_dim;
		
		settings >> s;
		settings >> m_maxIterations;
		
		settings >> s;
		settings >> m_representationUpdate;
		
		settings >> s;
		settings >> m_maxNodes;
		
		settings >> s;
		settings >> m_insertEvery;
		
		settings >> s;
		settings >> m_newNodePos;
		
		settings >> s;
		settings >> m_errorDecayInsert;
		
		settings >> s;
		settings >> m_ageMax;
		
		settings >> s;
		settings >> m_moveWinner;
		
		settings >> s;
		settings >> m_moveNeighbor;
		
		settings >> s;
		settings >> m_errorDecay;
		
		settings >> s;
		settings >> m_utilityDecay;
		
		settings >> s;
		settings >> m_utilityBound;
		
		settings >> s;
		settings >> m_distributionType;
		
		settings >> s;
		settings >> m_iterationDelay;
	}

*/
