/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __GNGSETTINGS_H
#define __GNGSETTINGS_H

#include <iostream>
#include <fstream>
#include <string>

/**
 * A class that reads the ini-file and then contains the GNG parameters.
 */
class Settings
{
public:
  /**
   * @param filename The filename of the ini-file to read.
   */
  Settings(std::string filename);

protected:
  void SkipComment(std::ifstream & s);

public:      
      std::string m_mode;   // (gng, gngu, sgng )
      unsigned int m_dim;                  /// dimension of input and reference vectors
      bool         m_initFromDistribution; // initialize the two first node-positions from input distr.
      unsigned int m_maxIterations;        /// a limit for number of iterations.

      unsigned int m_representationUpdate; /// a limit for number of iterations.
      
      unsigned int m_maxNodes;     /// Max Nodes
      unsigned int m_insertEvery;  /// Lambda
      double m_newNodePos;         /// --not included in the Original GNG description
      double m_errorDecayInsert;   /// Alfa
      
      double m_ageMax;             /// Age Max
      double m_moveWinner;         /// Epsilon Winner
      double m_moveNeighbor;       /// Epsilon Neighbor
      double m_errorDecay;         /// Beta      
      double m_utilityDecay;       ///  --not included in the Original GNG description. SAME as Beta.
      double m_utilityBound;       /// U - Utility for GNG-U

      unsigned int m_GNGMSEBackLog;
 
      int          m_distributionType; // choose from the different types of distributions
      unsigned int m_iterationDelay;   // delay in milliseconds between each iteraton.
      double       m_RBFbackLog;          //MISCLASSIFICATION_BACKLOG
      unsigned int m_maxMisclassification; //NUM_MISCLASSIFICATIONS
      double       m_classificaitonDecisionLimit;
      double       m_classificaitonDecisionDelta;
      double       m_maxSquaredError;  //max allowed squared error for function approximation

      double       m_eta; // step-size used in weight training (sgng)
};


#endif
