/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include "Vector.hpp"
#include <cmath>
#include <iostream>

using namespace std;

//----------
double Vector::SquaredDistance(Vector const & v) const
{
  if( v.size() != this->size() )
    throw Exception("Vector SquaredDistance:: vector has incorrect size.");
  
  Vector const & diffVector = *this - v;

  double result = 0.0;
  for(unsigned int i=0; i<v.size(); i++)
    result += diffVector[i] * diffVector[i];

  return result;
}


//----------
double Vector::Distance(Vector const & v) const
{
  return sqrt( SquaredDistance(v) ); 
}


//----------
Vector operator-(Vector const & u, Vector const & v)
{
  if( v.size() != u.size() )
    throw Exception("Vector operator-:: vectors are of different size.");
  
  Vector newVector(v.size());
  
  for(unsigned int i=0; i<v.size(); i++)
    newVector[i] = u[i] - v[i];
  
  return newVector;
}


//----------
Vector operator+(Vector const & u, Vector const & v)
{
  if( v.size() != u.size() )
    throw Exception("Vector operator+:: vectors are of different size.");
  
  Vector newVector(v.size());
  
  for(unsigned int i=0; i<v.size(); i++)
    newVector[i] = u[i] + v[i];
  
  return newVector;
}

Vector operator+= (Vector & u, Vector const & v)
{
  if( v.size() != u.size() )
    throw Exception("Vector operator+=:: vectors are of different size.");
  
  for(unsigned int i=0; i<v.size(); i++)
    u[i] += v[i];
  
  return u;
}


//----------
Vector operator* (Vector const & v, double d)
{
  Vector newVector(v.size());
  
  for(unsigned int i=0; i<v.size(); i++)
    newVector[i] = v[i] * d;

  return newVector;
}



//----------
ostream & operator<< (ostream & o, Vector const & v)
{
	Vector::const_iterator it = v.begin();
  //o << "[ ";
	for(; it != v.end(); it++)
	  o << *it << " ";
	//o << "]";
	return o;
}

