/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __VECTOR_HH
#define __VECTOR_HH

#include <vector>
#include <iostream>
#include "Exception.hpp"

/**
 * A Vector of doubles that is built on std::vector.
 */
class Vector : public std::vector<double>
{
public:
  /**
   * Creates an empty Vector.
   */
  Vector(){}
  /**
   * @param size The number of pre-allocated elements in the new Vector.
   */
  Vector(unsigned int size) : std::vector<double>(size) {}

  /**
   * Calculates the Squared euklidian distance between this Vector and the 
   * supplied Vector v, ||this - v||^2.
   * @param v a Vector.
   * @return The squared distance to v.
   */
  double SquaredDistance(Vector const & v) const;
  
  /**
   * Calculates the euklidian distance between this Vector and the 
   * supplied Vector v, ||this - v||.
   * @param v a Vector.
   * @return The distance to v.
   */
  double Distance(Vector const & v) const;

  friend Vector operator-  (Vector const & u, Vector const & v);
  friend Vector operator+  (Vector const & u, Vector const & v);
  friend Vector operator+= (Vector & u, Vector const & v);
  friend Vector operator*  (Vector const & u, double d);
  friend std::ostream & operator<< (std::ostream & o, Vector const & v);

};

#endif
